w, h = love.graphics.getDimensions()

shine = require "shine"
require("misc")
require("wave")
require("tableSer")
require("sound")

groundLevel = 1000

math.randomseed(os.time())

function love.load()
	love.graphics.setBackgroundColor(0, 0, 0)
	objects = {}

	split = shine.separate_chroma()
	split.parameters = {radius = 2, angle = 0}
	vignette = shine.vignette()
    vignette.parameters = {radius = 0.995, opacity = 0.95}

    -- World Constants
    world = {
    	grav = 200,
	}

	-- Player variables
	player = {
		id = createShip(white, grey),

		x = 0,
		y = 0,
		r = 0,
		rotSpeed = math.pi,

		vx = 0,
		vy = 0,
		speed = 500,
		normalSpeed = 500,
		maxSpeed = 1500,

		weapon = "machinegun",

		fric  = 0.4, -- Can be affected by mass/glide

		mid = {
			x = 0,
			y = 0,
		},
		
		bullets = {},
		timer = 0.1,
		maxTimer = 0.1,
		bulletSpeed = 1500,
		bulletLength = 300,

		speedTimer = 0,
		soundID = 0,
	}

	-- Camera functions
	camera = {
		xx = function() return (player.mid.x + player.x)*0.5 - w/2 + shake end,
		x = function() return player.x - w/2 end,
		yy = function() return (player.mid.y + player.y)*0.5 - h/2 + shake end,
		y = function() return player.y - h/2 end,
	}

	-- Spawn reference points
	rect = {}
	for i=1,1000 do
		rect[i] = {
			x = math.random(-1000, 1000),
			y = math.random(-1000, 1000-16-5),
		}
	end

	-- Spawn enemies
	enemies = {}
	eBullets = {} -- Enemy bullets
	spawnEnemies(5)

	shake = 0

	newSound("sfx/hit.wav")
	newSound("sfx/die.wav")
	newSound("sfx/laser.wav", true)
	newSound("sfx/deflect.wav")

	--addWave(math.pi)
	--addExplosion(player.x, player.y, 128, 20)
end

function love.update(dt)
	-- Mouse positions
	local x, y = love.mouse.getPosition()
	x = x + camera.x(); y = y + camera.y()
	
	-- Turn the ship
	local mouseAng = angleDifference(player.r, math.angle(player.x, player.y, x, y))
	player.r = player.r - math.sign(mouseAng) * player.rotSpeed * dt
	-- Calibrate the angle
	player.r = calibrateAng(player.r)
	-- Lock to mouse angle
	if math.abs(math.round(math.deg(mouseAng))) <= 2 then
		player.r = math.angle(player.x, player.y, x, y)
	end

	-- Apply velocity
	local ax, ay = 0, 0
	if (love.keyboard.isDown("space") or love.keyboard.isDown(" ")) and velocity(player) < player.maxSpeed then
		player.speed = player.speed + player.normalSpeed * dt
	end
	if not (love.keyboard.isDown("space") or love.keyboard.isDown(" ")) then
		player.speed = player.normalSpeed
	end

	if love.keyboard.isDown("w") and velocity(player) < player.maxSpeed then
		ax = math.cos(player.r) * player.speed
		ay = math.sin(player.r) * player.speed
	end
	ax = ax - player.vx * player.fric
	player.vx = player.vx + (ax * dt)

	if player.y < groundLevel then
		player.vy = player.vy + (ay * dt) + (world.grav * dt)
	else
		if player.vy > -(world.grav * 50 * dt) then
			player.vy = player.vy - (world.grav * 50 * dt)
		end
	end

	player.x = player.x + player.vx * dt
	player.y = player.y + player.vy * dt
	-- Hard limit
	if player.y > groundLevel then player.y = groundLevel end
	---------------------------------------------------------

	for i,v in ipairs(enemies) do
		if v.timer < v.maxTimer then v.timer = v.timer + dt end
		-- Turn the ship
		local err = math.rad(math.random(-2, 2)) -- Error in locating player
		local ang = 0 

		if v.target == "player" then
			ang = angleDifference(v.r, math.angle(v.x, v.y, player.x, player.y) + err)
		else
			ang = angleDifference(v.r, math.rad(-90))
		end

		v.r = v.r - math.sign(ang) * v.rotSpeed * dt
		-- Calibrate the angle
		v.r = calibrateAng(v.r)
		-- Lock to angle
		if math.abs(math.round(math.deg(ang))) <= 2 then
			if v.target == "player" then
				v.r = math.angle(v.x, v.y, player.x, player.y) + err

				if v.timer >= v.maxTimer then
					local k = {
						x = v.x,
						y = v.y,
						r = v.r,

						speed = v.bulletSpeed,

						t = 2,
					}
					table.insert(eBullets, k)

					v.timer = 0
				end
			else
				v.r = math.rad(-90)
			end
		end

		local ax, ay = 0, 0
		if player.y > v.y and math.dist(player.x, player.y, v.x, v.y) < h and v.vy > v.speed/2 then
			v.target = "brake"
		else
			v.target = "player"
		end

		if velocity(v) < v.maxSpeed then
			ax = math.cos(v.r) * v.speed
			ay = math.sin(v.r) * v.speed
		end
		ax = ax - v.vx * v.fric
		v.vx = v.vx + (ax * dt)

		if v.y < groundLevel then
			v.vy = v.vy + (ay * dt) + (world.grav * dt)
		else
			if v.vy > -(world.grav * 50 * dt) then
				v.vy = v.vy - (world.grav * 50 * dt)
			end
		end

		v.x = v.x + v.vx * dt
		v.y = v.y + v.vy * dt
		-- Hard limit
		if v.y > groundLevel and v.y > v.speed*1.4 then table.remove(enemies, i); playSound("die")
		elseif v.y > groundLevel then v.y = groundLevel end
	end

	-- RGB split
	if love.mouse.isDown(1) or love.mouse.isDown("l") then
		if player.weapon == "beam" then
			split.parameters = {radius = math.random(2,4), angle = math.random(0, 2*math.pi)}
			shake = math.random(-5, 5)

			if player.soundID == 0 then player.soundID = playSound("laser") end

			for i,v in ipairs(enemies) do
				if player.weapon == "beam" then
					if math.lineCircleIntersect(player.x,player.y,player.x + math.cos(player.r) * w,player.y + math.sin(player.r) * w, v.x,v.y,16*math.sqrt(2) + 4) then
						if math.abs(angleDifference(player.r, math.angle(player.x, player.y, v.x, v.y))) < math.rad(10) then
							--playSound("die")
							addExplosion(v.x, v.y, math.random(64, 160), math.random(200, 240))

							table.remove(enemies, i)
							split.parameters = {radius = math.random(5,6), angle = player.r}
						end
					end
				end
			end
		elseif player.weapon == "machinegun" then
			if player.timer >= player.maxTimer then
				local i = {
					x = player.x,
					y = player.y,
					r = player.r,
					t = 2,
				}
				table.insert(player.bullets, i)

				playSound("hit")

				split.parameters = {radius = math.random(2,3), angle = player.r}
				shake = math.random(-2, 2)

				player.timer = 0
			end
			if player.soundID ~= 0 then stopSound("laser", player.soundID); player.soundID = 0 end
		end
	elseif not love.mouse.isDown(1) then
		split.parameters = {radius = 1, angle = player.r}
		if player.soundID ~= 0 then stopSound("laser", player.soundID); player.soundID = 0 end
	end

	if player.timer < player.maxTimer then player.timer = player.timer + dt end
	for i,v in ipairs(player.bullets) do
		v.t = v.t - dt
		v.x = v.x + math.cos(v.r) * player.bulletSpeed * dt
		v.y = v.y + math.sin(v.r) * player.bulletSpeed * dt

		for j,k in ipairs(enemies) do
			if math.dist(v.x, v.y, k.x, k.y) <= 16*math.sqrt(2) then
				playSound("die")
				addExplosion(v.x, v.y, math.random(48, 96), math.random(200, 240))

				table.remove(enemies, j)
				table.remove(player.bullets, i)
				break
			end
		end

		if v.t <= 0 then table.remove(player.bullets, i) end
	end

	for i,v in ipairs(eBullets) do
		v.t = v.t - dt
		v.x = v.x + math.cos(v.r) * v.speed * dt
		v.y = v.y + math.sin(v.r) * v.speed * dt

		if math.dist(v.x, v.y, player.x, player.y) <= 16*math.sqrt(2) then
			shake = math.random(-3, 3)
			split.parameters = {radius = math.random(5,6), angle = v.r}

			addWave(math.atan2(v.y - player.y, v.x - player.x))

			playSound("deflect")
			table.remove(eBullets, i)
			break
		end

		if v.t <= 0 then table.remove(eBullets, i) end
	end

end

function love.draw()
	vignette:draw(function()
	split:draw(function()
	love.graphics.setColor(black)
	love.graphics.rectangle("fill", 0, 0, w, h)
	--player.mid.x = math.cos(player.r - math.pi/2) * (w/2 - player.w/2) + player.x
	--player.mid.y = math.sin(player.r - math.pi/2) * (h/2 - player.h/2) + player.y
	player.mid.x, player.mid.y = love.mouse.getPosition()
	player.mid.x = player.mid.x + camera.x()
	player.mid.y = player.mid.y + camera.y()

	love.graphics.push()
	love.graphics.translate(-camera.xx(), -camera.yy())

	-- Reference points
	love.graphics.setColor(green)
	for i,v in ipairs(rect) do
		love.graphics.rectangle("fill", v.x-5, v.y-5, 10, 10)
	end

	-- Shadow and Sea Level
	love.graphics.setScissor(0, groundLevel-16 - camera.yy(), w, 1000)
	drawShip(player, player.x, player.y, true)
	for i,v in ipairs(enemies) do drawShip(v, v.x, v.y, true) end

	love.graphics.push()
		love.graphics.translate(player.x, groundLevel+16)
		love.graphics.scale(1, -1)
		love.graphics.translate(-player.x, -groundLevel-16)

		if love.mouse.isDown(1) or love.mouse.isDown("l") then
			if player.weapon == "beam" then
				love.graphics.setColor(grey)
				love.graphics.setLineWidth(8)
				love.graphics.line(player.x, player.y, player.x + math.cos(player.r) * w, player.y + math.sin(player.r) * w)
				love.graphics.setLineWidth(1)
			end
		end
		love.graphics.setColor(grey)
		for i,v in ipairs(player.bullets) do love.graphics.rectangle("fill", v.x-3, v.y-3, 6, 6) end
		love.graphics.setColor(lightblue)
		for i,v in ipairs(eBullets) do love.graphics.rectangle("fill", v.x-3, v.y-3, 6, 6) end

		drawExplosions(grey)
	love.graphics.pop()
	love.graphics.setScissor()

	-- Normal Draws
	love.graphics.setColor(white)
	love.graphics.translate(camera.xx(), camera.yy())
		love.graphics.line(0, groundLevel-16 - camera.yy(), w, groundLevel-16 - camera.yy())
	love.graphics.translate(-camera.xx(), -camera.yy())

	drawForceField(player.x, player.y, "back")
	drawShip(player, player.x, player.y)
	drawForceField(player.x, player.y, "front")

	for i,v in ipairs(enemies) do
		drawShip(v, v.x, v.y)
	end

	love.graphics.setColor(white)
	--[[love.graphics.line(player.x, player.y, player.mid.x, player.mid.y)
	love.graphics.rectangle("fill", player.mid.x-3, player.mid.y-3, 6, 6)]]

	for i,v in ipairs(player.bullets) do love.graphics.rectangle("fill", v.x-3, v.y-3, 6, 6) end
	love.graphics.setColor(blue)
	for i,v in ipairs(eBullets) do love.graphics.rectangle("fill", v.x-3, v.y-3, 6, 6) end

	love.graphics.setColor(white)
	if love.mouse.isDown(1) or love.mouse.isDown("l") then
		if player.weapon == "beam" then
			love.graphics.setLineWidth(8)
			love.graphics.line(player.x, player.y, player.x + math.cos(player.r) * w, player.y + math.sin(player.r) * w)
			love.graphics.setLineWidth(1)
		end
	end

	drawExplosions()

	love.graphics.pop()
	end)
	end)

	love.graphics.setColor(white)
	love.graphics.print(string.format("FPS: %1.f\nNumber of Enemies: %1.f\nx: %1.f\ny: %1.f\nr: %1.f\nv: %1.f", love.timer.getFPS(), #enemies, player.x, player.y, math.deg(player.r), velocity(player)))
	love.graphics.setColor(white)
end

function love.keypressed(key)
	if key == "escape" then love.event.quit() end
	if key == "q" then spawnEnemies(100) end
	if key == "e" then spawnEnemies(5) end
	if key == "1" then player.weapon = "beam" end
	if key == "2" then player.weapon = "machinegun" end
end