function point3DtoIso(a) return {a[1] + w/2, a[2] + h/2} end

black = {0, 0, 0}
white = {255, 255, 255}
grey  = {40, 40, 40}

red = {255, 80, 80}
lightred = {255, 100, 100}

green = {80, 255, 80}
lightgreen = {100, 255, 100}

blue = {80, 80, 255}
lightblue = {100, 100, 255}

function createShip(c1, c2)
	local id = #objects + 1
	objects[id] = {
		nodes = {},
		faces = {},
	}
	-- 32x32
	newNode(0, 0, 0, id)
	newNode(0, 0, -16, id)
	newNode(0, -16, 16, id)
	newNode(-16, 0, 16, id)
	newNode(16, 0, 16, id)

	newFace({2, 3, 4}, id, c1, c2)
	newFace({2, 3, 5}, id, c1, c2)

	return id
end

function newNode(x, y, z, id)
	table.insert(objects[id].nodes, {
		[1] = x,
		[2] = y, 
		[3] = z,
		x = x,
		y = y,
		z = z,
	})
end

function newFace(vertices, id, color, color2)
	table.insert(objects[id].faces, {vert = vertices, c = color, c2 = color2})
end

function removeFace(id, i)
	table.remove(objects[id].faces, i)
end

function setObjectAng(id, ang, axis)
	local k = objects[id]
	local j = k.nodes[1]

	local sin_t = math.sin(ang)
	local cos_t = math.cos(ang)
	 
	for i,v in ipairs(k.nodes) do
	   	if i ~= 1 then
			local x = v.x - j.x
			local y = v.y - j.y
			local z = v.z - j.z

			if axis == "z" then -- Around (circle)
				v[1] = x * cos_t - y * sin_t + j.x
        		v[2] = y * cos_t + x * sin_t + j.y
			elseif axis == "y" then -- Left Right
				v[1] = x * cos_t - z * sin_t + j.x
            	v[3] = z * cos_t + x * sin_t + j.z
			elseif axis == "x" then -- Up Down
				v[2] = y * cos_t - z * sin_t + j.y
           		v[3] = z * cos_t + y * sin_t + j.z
			end
		end
	end
end

local ref = {
	[1] = "x",
	[2] = "y",
	[3] = "z",
}
function setObjectPos(id, pos)
	local k = objects[id]
	local j = k.nodes[1]

	if j[1] ~= pos[1] or j[2] ~= pos[2] or j[3] ~= pos[3] then
		local newJ = {
			[1] = j[1] + (pos[1] - j[1]),
			[2] = j[2] + (pos[2] - j[2]),
			[3] = j[3] + (pos[3] - j[3]),
			idd = -1,
			x = j[1] + (pos[1] - j[1]),
			y = j[2] + (pos[2] - j[2]),
			z = j[3] + (pos[3] - j[3]),
		}

		for i,v in ipairs(k.nodes) do
			if i ~= 1 then
				for n=1,#v do
					v[n] = v[ref[n]]
					v[n] = newJ[n] + v[n] - j[n]
					
					v[ref[n]] = v[n]
				end
			end
		end

		k.nodes[1] = newJ
	end
end

function drawShip(t, x, y, inv)
	inv = inv or false
	x, y = x - w/2, y - h/2

	setObjectPos(t.id, {x, y, 0})

	local angg = t.r
	if math.abs(math.deg(angg)) > 90 then angg = angg - (angg - math.sign(angg)*(math.pi/2))*2 end
	setObjectAng(t.id, angg, "x")
	setObjectAng(t.id, t.r + math.rad(90), "y")
	
	love.graphics.push()
	if inv then
		love.graphics.translate(t.x, groundLevel+16)
		love.graphics.scale(1, -1)
		love.graphics.translate(-t.x, -groundLevel-16)

		if t.mid then -- Player
			drawForceField(player.x, player.y, "back", lightred)
		end
	end

	for k,j in ipairs(objects[t.id].faces) do
		local points = {}
	
		for q=1, #j.vert do
			local n = j.vert[q]
			local node = point3DtoIso(objects[t.id].nodes[n])
				
			points[#points + 1] = node[1]
			points[#points + 1] = node[2]
		end

		if inv then
			love.graphics.setColor(j.c2)
		else
			love.graphics.setColor(j.c)
		end

		love.graphics.polygon("fill", points)
	end

	if inv and t.mid then -- Player
		drawForceField(player.x, player.y, "front", lightred)
	end

	love.graphics.pop()
end

function calibrateAng(r)
	if r > math.pi then
		return -math.pi + (r - math.pi)
	elseif r < -math.pi then
		return math.pi + (r + math.pi)
	end

	return r
end

function spawnEnemies(n)
	for i = 1,n do
		local i = {
			id = createShip(blue, lightblue),

			x = math.random(-1000, 1000),
			y = math.random(-1000, 1000-16),
			r = 0,
			rotSpeed = math.pi*2,

			vx = 0,
			vy = 0,
			speed = 800,
			maxSpeed = 1500,

			fric = 0.4,

			target = "player",

			timer = 10,
			maxTimer = math.random(0.2, 0.3),
			bulletSpeed = math.random(1000, 2000),
		}
		table.insert(enemies, i)
	end
end

function velocity(t) return math.sqrt(t.vx^2 + t.vy^2) end

-- Returns 'n' rounded to the nearest 'deci'th (defaulting whole numbers).
function math.round(n, deci) deci = 10^(deci or 0) return math.floor(n*deci+.5)/deci end
-- Returns the angle between two points.
function math.angle(x1,y1, x2,y2) return math.atan2(y2-y1, x2-x1) end
function angleDifference(t1, t2) return (t1 - t2 + math.pi) % (math.pi * 2) - math.pi end
-- Returns 1 if number is positive, -1 if it's negative, or 0 if it's 0.
function math.sign(n) return n>0 and 1 or n<0 and -1 or 0 end
-- Returns the distance between two points.
function math.dist(x1,y1, x2,y2) return ((x2-x1)^2+(y2-y1)^2)^0.5 end
-- Checks if a line intersects a circle
function math.lineCircleIntersect(x1,y1,x2,y2, x0,y0,r)
	local a = y2-y1
	local b = x2-x1
	local nn = math.abs(a*x0 - b*y0 + x2*y1 - y2*x1)
	local mm = math.sqrt(a^2 + b^2)
	return (nn/mm) <= r
end